/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ste.tmc.meldungserstellung;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DynamicObject;
import de.bsvrz.dav.daf.main.config.InvalidationListener;
import de.bsvrz.dav.daf.main.config.MutableSet;
import de.bsvrz.dav.daf.main.config.MutableSetChangeListener;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Abtrakte Klasse zur Verwaltung von dynamischen DAV-Objekten. Diese abstrakte
 * Klasse stellt Funktionalität zur Vefügung, um generische Instanzen für
 * dynamische DAV-Objekte, die in einer dynamischen Menge enthalten sind, zu
 * verwalten.<br>
 * Die verwalteten Instanzen werden in einer (nicht modifizierbaren) Map mit der
 * PID des dynamischen Objekts als Schlüssel zur Verfügung gestellt. Diese Map
 * wird entsprechend den Änderungen in der dynamischen Objektmenge und den
 * Löschungen (Invalidierungen) der dynamischen Objekte aktualisiert.<br>
 * Zur Implementierung einer effektiven Verwaltung genügt die Implementierung
 * der Methode <code>erzeugeObjekt</code> und aufrufen der Methode
 * <code>initialisiere</code> z. B. im Konstruktor der beerbenden Klasse.
 * 
 * 
 * Erstellt auf Basis der SWE RDS/TMC von: Dambach Werke GmbH, Stefan Sans
 * 
 * @since 07/2016: Adaptiert für TMC-Meldungen durch: BitCtrl Systems GmbH
 * @author BitCtrl Systems GmbH, Gieseler
 */
public abstract class AbstraktVewDynObj<T> implements MutableSetChangeListener, InvalidationListener {
	/**
	 * Debug-Logger für Logging-Ausgaben.
	 */
	private static final Debug DEBUG = Debug.getLogger();

	/**
	 * Die verwalteten Instanzen vom Typ <code>T</code>.
	 */
	private Map<String, T> verwalteteObjekte = new HashMap<String, T>();

	/**
	 * Read-only Zugriff auf die verwalteten Instanzen vom Typ <code>T</code>.
	 * 
	 * @return Map (nicht modifizierbar) der verwalteten Objekte, Schlüssel ist
	 *         die PID des dynamischen Objekts
	 */
	public Map<String, T> getVerwalteteObjekte() {
		return Collections.unmodifiableMap(verwalteteObjekte);
	}

	/**
	 * Erzeuge eine neue Instanz vom verwalteten Objekttyp <code>T</code>.
	 * 
	 * @param sysObj
	 *            das dynamische DAV-Objekt, zu dem die Instanz vom Typ
	 *            <code>T</code> erzeugt wird
	 * 
	 * @return das erzeugte Objekt vom Typ <code>T</code>
	 */
	protected abstract T erzeugeObjekt(SystemObject sysObj);

	/**
	 * Muss aufgerufen werden, um die Verwaltung für die aktuell in der
	 * dynamischen Menge enthaltenen Objekten zu initialisieren - z. B. als
	 * Bestandteil des Konstruktors.
	 * 
	 * @param konfigurationsObjekt,
	 *            das Besitzer der dynamischen Menge ist
	 * @param mengenBezeichner
	 *            Bezeichner (Name) der Menge
	 */
	protected void initialisiere(final ConfigurationObject konfigurationsObjekt, final String mengenBezeichner) {
		final MutableSet menge = konfigurationsObjekt.getMutableSet(mengenBezeichner);
		if (menge != null) {
			int anzahl = 0;

			for (SystemObject obj : menge.getElements()) {
				DEBUG.finer("Lese Objekt: " + obj + "\t" + obj.getType() + "\t" + obj.isValid());
				if (obj.isValid()) {
					if (initialisiereObjekt(obj) != null) {
						anzahl++;
					}
				}
			}

			DEBUG.config("Verwalte " + anzahl + " Element(e) aus der dynamischen Menge \"" + mengenBezeichner
					+ "\" von " + konfigurationsObjekt);

			menge.addChangeListener(this);
		} else {
			throw new IllegalArgumentException(
					"Dynamische Menge \"" + mengenBezeichner + "\" nicht vorhanden in " + konfigurationsObjekt);
		}
	}

	/**
	 * Wird von der Verwaltung aufgerufen und führt Initialisierungen durch,
	 * wenn ein dynamisches Objekt erzeugt oder der dynamischen Menge
	 * hinzugefügt wird.
	 * 
	 * @param sysObj
	 *            das dynamische Object, das hinzugefügt wird
	 * @return die Instanz vom Typ <code>T</code>, für die die Verwaltung
	 *         initialisiert wurde - für diese Instanz können jetzt ggf. weitere
	 *         Resourcen belegt werden. Wenn die Initialisierung nicht
	 *         ergolgreich war wird <code>null</code> zurückgegenen
	 */
	protected T initialisiereObjekt(final SystemObject sysObj) {
		if (verwalteteObjekte.get(sysObj.getPid()) == null) {
			final T obj = erzeugeObjekt(sysObj);

			if (obj != null) {
				synchronized (this) {
					verwalteteObjekte.put(sysObj.getPid(), obj);
					if (sysObj instanceof DynamicObject) {
						((DynamicObject) sysObj).addListenerForInvalidation(this);
					}
				}
			}

			return obj;
		} else {
			DEBUG.warning("Objekt mit PID " + sysObj.getPid() + " wird bereits verwaltet");
			return null;
		}
	}

	/**
	 * Wird von der Verwaltung aufgerufen und führt Bereinigungen durch, wenn
	 * ein dynamisches Ovjekt gelöscht oder aus der dynamischen Menge entfernt
	 * wird.
	 * 
	 * @param sysObj
	 *            das dynamische Object, das entfernt wird
	 * @return die Instanz vom Typ <code>T</code>, für das die Verwaltung
	 *         bereinigt wurde - für diese Instanz können jetzt ggf. weiter
	 *         Resourcen freigegeben werden. Wenn die Bereinigung nicht
	 *         erfolgreich war wird <code>null</code> zurückgegenen
	 */
	protected T bereinigeObjekt(final SystemObject sysObj) {
		T obj = null;

		if (sysObj instanceof DynamicObject) {
			((DynamicObject) sysObj).removeListenerForInvalidation(this);
		}

		synchronized (this) {
			if (verwalteteObjekte.get(sysObj.getPid()) != null) {
				obj = verwalteteObjekte.remove(sysObj.getPid());
			}
		}

		return obj;
	}

	/*
	 * (Kein Javadoc)
	 * 
	 * @see
	 * de.bsvrz.dav.daf.main.config.MutableSetChangeListener#update(de.bsvrz.dav
	 * .daf.main.config.MutableSet, de.bsvrz.dav.daf.main.config.SystemObject[],
	 * de.bsvrz.dav.daf.main.config.SystemObject[])
	 */
	public void update(final MutableSet set, final SystemObject[] addedObjects, final SystemObject[] removedObjects) {
		DEBUG.finer("MutableSetChangeListener.update() von \"" + set.getName() + "\"");

		if (addedObjects.length > 0) {
			DEBUG.config(
					"Dynamische Menge \"" + set.getName() + "\" um " + addedObjects.length + " Objekte vergrössert");

			for (SystemObject obj : addedObjects) {
				initialisiereObjekt(obj);
			}
		}

		if (removedObjects.length > 0) {
			DEBUG.config(
					"Dynamische Menge \"" + set.getName() + "\" um " + removedObjects.length + " Objekte verkleinert");

			for (SystemObject obj : removedObjects) {
				bereinigeObjekt(obj);
			}
		}
	}

	/*
	 * (Kein Javadoc)
	 * 
	 * @see
	 * de.bsvrz.dav.daf.main.config.InvalidationListener#invalidObject(de.bsvrz.
	 * dav.daf.main.config.DynamicObject)
	 */
	public void invalidObject(final DynamicObject dynamicObject) {
		DEBUG.config("Objekt " + dynamicObject + " gelöscht/invalidiert => wird nicht weiter verwaltet");

		bereinigeObjekt(dynamicObject);
	}

}
